from pathlib import Path

import selectolax
from loguru import logger


def _create_folder(folder_name: str):
	"""Create a folder with the name `folder_name` if it doesn't exist yet

	:param folder_name:
	:type folder_name: str
	:return: Path
	:rtype: Path object
	"""
	logger.info(f"Create folder with {folder_name} as name if not exists yet ...")
	Path(folder_name).mkdir(exist_ok=True)
	return Path(folder_name)


def scripts_hunter(css_selector: str, parser_instance: selectolax.parser.HTMLParser):
	div_with_class_container_or_id_root = parser_instance.css_first(css_selector)
	if div_with_class_container_or_id_root is None:
		logger.warning(
			"Something went wrong with the parsing, an empty none value has been returned "
		)
		return None
	list_scripts = div_with_class_container_or_id_root.css("script")
	# print(list_scripts)
	json_result = ""
	for script in list_scripts:
		script_content = script.text()
		# print(script_content)
		if "window.__page__data =" in script_content:
			# print(script_content)
			json_result = (
				script.text()
				.replace("window.__page__data__config =", "")
				.replace(
					"window.__page__data = window.__page__data__config.props",
					"",
				)
				.replace("window.__page__data =", "")
				.strip("\n \t \b ")
			)
			break
	return json_result


def json_hunter(html_content: str | bytes, css_selector: str = ""):
	"""Parse the HTML content of the page to get all json content in the divs where eitheir class=`.organic-list.viewtype-list` or  id='root'.

	:param html_content: The HTML content of the page
	:type html_content: str
	:return: A list of divs with class `.organic-list.viewtype-list`
	:rtype: list
	"""
	body_parser = selectolax.parser.HTMLParser(html_content)
	json_result = scripts_hunter(css_selector, body_parser)
	if json_result == "":
		json_result = scripts_hunter("div[id='root']", body_parser)
		return json_result
	if json_result is None:
		return None
	return json_result.strip()


@logger.catch()
def write_to_disk(folder_name: str, pages_contents: list[str]):
	"""Save the content of each page as a separate HTML file in a folder named `folder_name`.

	:param folder_name: The name of the folder where the HTML pages will be saved
	:type folder_name: str
	:param pages_contents: A list of strings containing the HTML content of each page
	:type pages_contents: list[str]
	:return: None
	"""
	logger.info("Saving your scrapping result in disk...")
	path_obj = _create_folder(folder_name)
	for page_number, content in enumerate(pages_contents):
		with open(
			(path_obj / f"page_{page_number + 1}.html").resolve(),
			"w",
			encoding="utf-8",
		) as file:
			file.write(content)
		logger.info(f"Html content from page {page_number + 1} has been saved succesfully !")


if __name__ == "__name__":
	...



import json
from collections import OrderedDict
from dataclasses import dataclass
from pathlib import Path
from typing import Sequence, Union

import rich
from loguru import logger
from rich.progress import (
	Progress,
	SpinnerColumn,
)
from selectolax.parser import HTMLParser

from aba_cli_scrapper.typed_datas import ProductDict, SupplierDict

from .html_to_disk import json_hunter
from .utils_scrapping import (
	country_name,
	custom_minium_to_oder,
	get_product_certification,
	get_product_price,
	is_alibaba_guaranteed,
	is_customizable,
	is_full_promotion,
	is_instant_order,
	is_trade_product,
	ordered_or_sold,
	suppliers_status,
)


@dataclass
class PageParser:
	"""Parse each html page downloaded from alibaba and return a list of suppliers and products.

	:raises TypeError: _description_
	:raises FileNotFoundError: if the targeted folder does not exist
	:return: PageParser object
	:rtype: class `PageParser`
	"""

	# folder path that contains all html files to be scraped
	targeted_folder: Union[str, Path]

	def _retrieve_html_content_as_string(self, html_file: Path):
		"""Retrieve html content from a html file and return it as a string.

		:param html_file: The path of the html file to be read.
		:type html_file: str
		:return: The html content as a string.
		:rtype: str
		"""
		logger.info(f"Retrieving html content from file : {html_file}")
		html_content = html_file.read_text(
			encoding="utf-8",
		)
		logger.info("Html content has been retrieved.")
		return html_content

	@logger.catch(FileNotFoundError)
	def _html_files_explorer(self):
		"""
		Retrieves all the html files in the targeted folder.

		:raises FileNotFoundError: if the targeted folder does not exist
		:return: A list of all html files in the targeted folder
		:rtype: List[Path]
		"""
		targeted_folder = Path(self.targeted_folder).resolve()
		if not targeted_folder.exists():
			raise FileNotFoundError()
		logger.info(f"getting html files from {targeted_folder} ... ")
		html_files = [html_file for html_file in targeted_folder.glob("*.html")]
		logger.info("Html files list has been returned.")
		return html_files

	def _divs_and_dict(self, selector: str = ".fy23-search-card"):
		"""
		Retrieves the div tags that match the given selector and a dictionary of parsed content from the html file
		containing the div tag.

		:param selector: The css selector to retrieve the div tags.
		:type selector: str
		:return: A list of tuples containing the div tag and its corresponding parsed content.
		:rtype: List[Tuple[selectolax.Selector, Dict]]
		"""
		logger.info(f"Retrieving  div tags with class='{selector}' ...")
		divs_and_dict = [
			(
				HTMLParser(self._retrieve_html_content_as_string(html_file)).css(selector),
				json_hunter(
					self._retrieve_html_content_as_string(html_file),
					css_selector="div[class='container']",
				),
			)
			for html_file in self._html_files_explorer()
		]
		logger.info("All expected divs and dict has been retrived ...")

		new_divs_and_dict = []
		for item in divs_and_dict:
			if item[1] is None or item[1] == "":
				continue
			new_divs_and_dict.append((item[0], json.loads(item[1])))  # type: ignore

		return new_divs_and_dict

	def detected_suppliers(self):
		"""Retrieves the detected suppliers from the divs and offers data.
		Returns a list of unique suppliers with their relevant information.
		"""
		suppliers: Sequence[SupplierDict] = list()
		console = rich.console.Console()
		with Progress(
			SpinnerColumn(finished_text="[bold green]finished ✓[/bold green]"),
			*Progress.get_default_columns(),
			console=console,
			transient=True,
		) as progress:
			all_pages_task = progress.add_task(
				description="[blank]Parsing suppliers ...[/blank]",
			)
			# single_page_task = progress.add_task(description="Retrieving suppliers ...", total=1)
			for divs, offers in self._divs_and_dict():
				# print(offers)
				# print(type(offers))
				# with open("offers.json", "w") as f:
				#     json.dump(offers, f,indent=4)
				try:
					if offers["offerTotalCount"] == 0:
						continue
				except KeyError:
					if offers["props"]["offerTotalCount"] == 0:
						continue
				try:
					offers_values = offers["offerResultData"]["offers"]
				except KeyError:
					offers_values = offers["props"]["offerResultData"]["offers"]
					pass
					for offer in offers_values:
						suppliers.append({
							"name": offer["companyName"].lower(),
							"verified_type": suppliers_status(tags=divs, offer=offer),
							"sopi_level": offer["displayStarLevel"],
							"country_name": country_name(country_min=offer["countryCode"]),
							"gold_supplier_year": offer["goldSupplierYears"].split(" ")[0],
							"supplier_service_score": float(offer["supplierService"]),
						})
				# all_pages_task.
				progress.update(all_pages_task, advance=100 / len(self._divs_and_dict()))
		# removing supliers present twice in supliers dict
		unique_suppliers_tuple = list(OrderedDict((str(d), d) for d in suppliers).items())
		# print(unique_suppliers_tuple)
		unique_suppliers = [supplier_tuple[1] for supplier_tuple in unique_suppliers_tuple]
		return unique_suppliers

	def detected_products(self):
		"""Returns a list of unique products with their relevant information."""
		products: Sequence[ProductDict] = list()
		console = rich.console.Console()
		with Progress(
			SpinnerColumn(finished_text="[bold green]finished ✓[/bold green]"),
			*Progress.get_default_columns(),
			console=console,
			transient=True,
		) as progress:
			all_pages_task = progress.add_task(description="[blank]Parsing products ...[/blank]")
			for divs, offers in self._divs_and_dict():
				try:
					if offers["offerTotalCount"] == 0:
						continue
				except KeyError:
					if offers["props"]["offerTotalCount"] == 0:
						continue
				try:
					offers_values = offers["offerResultData"]["offers"]
				except KeyError:
					offers_values = offers["props"]["offerResultData"]["offers"]
					pass
					for offer in offers_values:
						products.append({
							"name": offer["enPureTitle"].lower(),
							"max_price": get_product_price(
								all_price_text=offer["price"], which="max"
							),
							"min_price": get_product_price(
								all_price_text=offer["price"], which="min"
							),
							"guaranteed_by_alibaba": is_alibaba_guaranteed(
								str_status=offer["halfTrust"]
							),
							"certifications": get_product_certification(offer=offer),
							"minimum_to_order": custom_minium_to_oder(
								offer["halfTrustMoq"].lower()
							),
							"ordered_or_sold": ordered_or_sold(offer=offer),
							"supplied_by": offer["companyName"].lower(),
							"product_score": float(offer["productScore"]),
							"review_count": float(offer["reviewCount"]),
							"review_score": float(offer["reviewScore"]),
							"shipping_time_score": float(offer["shippingTime"]),
							"is_full_promotion": is_full_promotion(
								str_status=offer["isFullPromotion"]
							),
							"customizable": is_customizable(str_status=offer["customizable"]),
							"instant_order": is_instant_order(
								str_status=offer["halfTrustInstantOrder"]
							),
							"trade_product": is_trade_product(str_status=offer["tradeProduct"]),
						})
				progress.update(all_pages_task, advance=100 / len(self._divs_and_dict()))
		unique_products_tuple = list(OrderedDict((str(d), d) for d in products).items())
		# removing supliers present twice in supliers dict
		unique_products = [product_tuple[1] for product_tuple in unique_products_tuple]
		return unique_products



"""This script is responsible for scraping data from the Alibaba website.
It uses the Playwright library to navigate through multiple pages
extract HTML content,and save it to disk. The script defines
functions to handle asynchronous tasks,interact with
browser contexts, and gather data from the scraped HTML files.
Additionally, it utilizes loguru for logging purposes and decouple for
managing environment variables.
"""

import asyncio
import os
from asyncio import TaskGroup
from typing import Optional

import playwright
import playwright.sync_api
import urllib3
from click import UsageError
from loguru import logger
from playwright.async_api import BrowserContext, async_playwright
from playwright.async_api import Page as AsyncPage
from playwright.sync_api import Error as PError
from playwright.sync_api import sync_playwright
from rich import print as rprint
from rich.progress import (
	Progress,
	SpinnerColumn,
	TaskID,
)

from . import SBR_WS_CDP_LIST
from .html_to_disk import write_to_disk
from .info_message import run_scrapper_with_success

HTML_PAGE_RESULT = []


async def goto_task(
	url: str,
	context_browser: BrowserContext,
	task: TaskID,
	progress: Progress,
	semaphore: asyncio.Semaphore,
	tg_instance: TaskGroup,
	page: AsyncPage,
	page_results: int,
) -> Optional[str]:
	"""Get HTML content from a URL.

	This function will load a page from a given URL,
	get its HTML content and save it to a global list.
	It will also update a progress bar with the task ID.

	Parameters:
		url (str): The URL of the page to be loaded.
		context_browser (BrowserContext): The browser context to be used.
		task (TaskID): The task ID of the current task.
		progress (Progress): The progress bar to be updated.
		semaphore (asyncio.Semaphore): The semaphore to be used to limit the number of concurrent tasks.
		tg_instance (TaskGroup): The task group instance.
		page (AsyncPage): The page object to be used.
		page_results (int): The number of pages to be scrapped.

	Returns:
		Optional[str]: The HTML content of the page if successful, otherwise None.
	"""
	try:
		async with page:
			logger.info(f"Loading page {url.split('page=')[1]} ... ")
			response = await page.goto(url, wait_until="domcontentloaded", timeout=0)
			if response is None:
				return None
			logger.info(f"get response text from web page {url.split('page=')[1]} ... ")
			html_body = await response.text()
			progress.start_task(task)
			progress.update(task, advance=100 / page_results)
			global HTML_PAGE_RESULT
			HTML_PAGE_RESULT.append(html_body)
			logger.info(f"Closing page {url.split('page=')[1]} ... ")
	except Exception as e:
		logger.error(f"Error processing page {url.split('page=')[1]}: {e}")
		return None


def urls_pusher(words: str, stop_at: int):
	for i in range(1, stop_at + 1):
		yield f"https://www.alibaba.com/trade/search?spm=a2700.galleryofferlist.0.0.64271d1dZRDxt1&fsb=y&IndexArea=product_en&keywords={words}&tab=all&&page={i}"


@logger.catch()
async def async_scrapper(*, save_in: str, key_words: str, page_results: int) -> None:
	"""
	Initiates scraping of Alibaba.com based on the provided keywords.

	:param save_in: The directory to store the raw HTML files.
	:type save_in: str
	:param key_words: The search term(s) for finding products on Alibaba. Enclose multiple keywords in quotes.
	:type key_words: str
	:param page_results: The number of pages to scrape. If omitted, 10 pages will be scraped.
	:type page_results: int

	:return: None
	:rtype: None

	:raises UsageError: If no internet connection is available.

	:raises SystemExit: If the Bright data account has been suspended by the system.

	:raises SystemExit: If no SCRAPING BROWSER API key is provided.

	:raises playwright._impl._errors.Error: If there is an error with the playwright browser.

	:raises Exception: If there is an error processing a page.
	"""
	async with async_playwright() as p:
		logger.info("Connecting to CDP and creating the browser... ")
		try:
			# print("rt")
			api_key = SBR_WS_CDP_LIST
			if api_key == "":
				rprint(
					"[red]You need to set your SCRAPING BROWSER API key from BrightData to Enable Async Scraping"
				)
				return
			browser = await p.chromium.connect_over_cdp(api_key)
		except urllib3.exceptions.NameResolutionError:
			rprint("[red]check your internet connection")
			return

		except playwright._impl._errors.Error as e:  # type: ignore
			if "Account is suspended" in str(e):
				# print(str(e))
				rprint(
					"[red]Bright data account has been suspended by system may your api is exhausted recharge it and try again. You can use `--sync-api` flag after your last command to enable sync scrapping but you may not encounter enought success.  [/red]"
				)
				return
			elif "exists" in str(e):
				# print(str(e))
				rprint(
					"[white] Seems like playwright is not installed. lets aba install it for you... [/white]"
				)
				os.system("playwright install")
			elif "WebSocket error" in str(e):
				rprint(
					"[red]Web Socket is disconnected. You May need to activate your Internet connexion"
				)
			else:
				rprint("[red]Unexpected error occured ...")

		context_browser = await browser.new_context()
		s_one = asyncio.Semaphore(value=10)
		logger.info("Loading pages results ... ")
		async with asyncio.Lock():
			async with asyncio.TaskGroup() as tg:
				with Progress(
					SpinnerColumn(finished_text="[bold green]finished ✓[/bold green]"),
					*Progress.get_default_columns(),
					transient=True,
				) as progress:
					task = progress.add_task("[green blink] async Scraping...", start=False)
					for url in urls_pusher(words=key_words, stop_at=page_results):
						async with asyncio.Lock():
							page = await context_browser.new_page()
						tg.create_task(
							goto_task(
								url=url,
								semaphore=s_one,
								progress=progress,
								task=task,
								tg_instance=tg,
								context_browser=context_browser,
								page=page,
								page_results=page_results,
							)
						)

		write_to_disk(save_in, HTML_PAGE_RESULT)
		run_scrapper_with_success(folder_name=save_in)


def sync_scrapper(*, save_in: str, key_words: str, page_results: int) -> None:
	# pages_urls = _looking_for_urls(keywords=key_words)
	with Progress(
		SpinnerColumn(finished_text="[bold green]finished ✓[/bold green]"),
		*Progress.get_default_columns(),
		transient=True,
	) as progress:
		task = progress.add_task(
			"[green blink] Sync Scraping...",
			start=False,
		)
		playwright = sync_playwright().start()
		try:
			browser = playwright.chromium.launch(headless=True)
		except playwright._impl._errors.Error:  # type: ignore
			rprint(
				"[white] Seems like playwright is not installed. lets aba install it for you... [/white]"
			)
			os.system("playwright install")
		context = browser.new_context()
		for url in urls_pusher(words=key_words, stop_at=page_results):
			page = context.new_page()
			logger.info(f"Loading page {url.split('page=')[1]} ... ")
			try:
				response = page.goto(url, wait_until="domcontentloaded", timeout=0)
				if response is None:
					return None
				logger.info(
					f"Returns the text representation of response body from page {url.split('page=')[1]} ... "
				)
				progress.start_task(task)
				html_content = response.text()
				progress.update(task, advance=100 / page_results)
				global HTML_PAGE_RESULT
				HTML_PAGE_RESULT.append(html_content)
				logger.info(f"Closing the page {url.split('page=')[1]} ... ")
				page.close()
			except PError as e:
				if "ERR_INTERNET_DISCONNECTED" in e.message:
					raise UsageError("Check your internet connection ... ")
	write_to_disk(save_in, HTML_PAGE_RESULT)
	run_scrapper_with_success(folder_name=save_in)
